/****************************************************************/
/*  FT-817 External Display(for Hardware test)                  */
/*                                                              */
/*  Author:    Hiroshi Shirahara(7J3AOZ)                        */
/*  Date:      2014/09/30                                       */
/*                                                              */
/*  This code is in the public domain.                          */
/*                                                              */
/*  This sketch requires the following libraries.               */
/*                                                              */
/*  Adafruit-RGB-LCD-Shield-Library                             */
/*  https://github.com/adafruit/Adafruit-RGB-LCD-Shield-Library */ 
/*                                                              */
/*  arduino_ft817                                               */
/*  https://github.com/8cH9azbsFifZ/arduino_ft817               */
/*                                                              */
/*  Metro-Arduino-Wiring                                        */
/*  https://github.com/thomasfredericks/Metro-Arduino-Wiring    */
/*                                                              */
/****************************************************************/

#include <Wire.h>                         // I2Cライブラリのヘッダーファイル
#include <Metro.h>                        // 「Metro」ライブラリのヘッダーファイル

#include <Adafruit_MCP23017.h>            // LCDシールドライブラリのヘッダーファイル
#include <Adafruit_RGBLCDShield.h>

#include <FT817.h>                        // FT-817ライブラリのヘッダーファイル

// ------- RGB LCDシールド関連定数定義

#define RED 0x1                           // LCDのバックライトが赤色
#define GREEN 0x2                         // LCDのバックライトが緑色
#define YELLOW 0x3                        // LCDのバックライトが黄色
#define BLUE 0x4                          // LCDのバックライトが青色
#define VIOLET 0x5                        // LCDのバックライトが紫色
#define TEAL 0x6                          // LCDのバックライトが水色
#define WHITE 0x7                         // LCDのバックライトが白色

// ------- FT-817ライブラリ関連定数

#define FT817_SPEED 38400                 // FT-817との通信ボーレート

// ------- ライブラリクラスインスタンス生成

// Metroライブラリのインスタンス生成
Metro timer1 = Metro(200);                // タイマー間隔を200msとして生成

// LCDシールドライブラリのインスタンス生成
Adafruit_RGBLCDShield lcd = Adafruit_RGBLCDShield();

// FT-817ライブラリのインスタンス生成
FT817 ft817(&Serial);                     // ハードウェアシリアルを指定して生成

// ------- グローバル変数定義

char rig_mode[5];                          // FT-817のモードを格納
long rig_freq;                             // FT-817の周波数を格納

int lcd_color;                             // 現在選択しているLCDのバックライト色

// ------ 初期化

void setup() {

  lcd_color = WHITE;                       // LCDのバックライト色の初期値を白色(7)にする
  
  lcd.begin(16, 2);                        // LCDシールド初期化
  lcd.setBacklight(lcd_color);             // LCDのバックライト色をセット

  lcd.setCursor(0,0);                      // LCDのカーソル位置を(0,0)にセット
  lcd.print("Connecting....");             // 起動メッセージを表示

  ft817.begin(FT817_SPEED);                // FT-817ライブラリ初期化
  delay(1000);                             // FT-817ライブラリ初期化待ち（1秒） 
  read_rig();                              // FT-817からのデータ読み込み  

  tone(8,400,100);                         // 起動を示すためにブザーを3回鳴らす
  lcd.clear();                             // LCD表示クリア

}

// ------ メインループ

void loop ()
{    

  uint8_t buttons;                          // LCDシールドのボタンデータを格納
  
  if (timer1.check() == 1) {                // タイマーが時間になったか？
    timer1.reset();                         // タイマーをリセット
    read_rig();                             // FT-817からのデータ読み込み
    display_data();                         // FT-817のデータ表示
  }

  buttons = lcd.readButtons();              // LCDシールドのボタンデータを取得

  if (buttons) {                            // どれかのボタンが押されているか？
    if (buttons & BUTTON_LEFT) {
      tone(8,400,100);                      // ボタンの押下を示すためにブザーを鳴らす
      lcd_color = lcd_color - 1;            // バックライト色を示す値を1減算する
      if (lcd_color < 1) lcd_color = 7;     // 減算した結果が1以下なら7にする
      }
    else if (buttons & BUTTON_RIGHT) {
      tone(8,400,100);                      // ボタンの押下を示すためにブザーを鳴らす
      lcd_color = lcd_color + 1;            // バックライト色を示す値を1加算する
      if (lcd_color > 7) lcd_color = 1;     // 加算した結果が7以上なら1にする
      }
    else {
      tone(8,200,100);                      // 間違ったボタンの押下を示すためにブザーを鳴らす
      }
    lcd.setBacklight(lcd_color);            // LCDのバックライト色をセット
    delay(200);                             // ボタンのチャタリング防止のため200msウェイト
    }
  
}

// ------ FT-817からのデータ読み込み

void read_rig ()
{
  
  while(1) {
    
    rig_freq = ft817.getFreqMode(rig_mode);  // FT-817から周波数とモードを読み込み
    if (rig_freq != 0) break;                // 正常に読み込めたらループを抜ける

    }

} 

// ------ FT-817からのデータ表示

void display_data ()
{

  long freq;                                 // 周波数データ計算用変数 
  int mhz, khz, hz;
  
  int filter_width;                          // フィルタバンド幅(0=WIDE,1=NARROW)
  char mode_buf[5];                          // モード表示用バッファ
  char disp_buf[17];                         // LCD表示用バッファ
  
  filter_width = 0;                          // バンド幅を初期化
  
  // ------- 得られた周波数データからMHz.kHz,Hzを求める
  
  freq = rig_freq * 10;                      // 得られた周波数をHz単位にする

  mhz = freq / 1000000;                      // MHzを得る
  khz = (freq % 1000000)/1000;               // KHzを得る 
  hz = freq % 1000;                          // Hzを得る

  // ------- 得られたモードを表示用に編集する

  if (strcmp("LSB",rig_mode) == 0) {
    strcpy(mode_buf,rig_mode);
    }
  else if (strcmp("USB",rig_mode) == 0) {
    strcpy(mode_buf,rig_mode);
    }
  else if (strcmp("CW",rig_mode) == 0) {
    strcpy(mode_buf,rig_mode);
    }
  else if (strcmp("CWR",rig_mode) == 0) {
    strcpy(mode_buf,rig_mode);
    }
  else if (strcmp("AM",rig_mode) == 0) {
    strcpy(mode_buf,rig_mode);
    }
  else if (strcmp("FM",rig_mode) == 0) {
    strcpy(mode_buf,rig_mode);
    }
  else if (strcmp("FMN",rig_mode) == 0) {
    strcpy(mode_buf,rig_mode);
    filter_width = 1;
    }
  else if (strcmp("DIG",rig_mode) == 0) {
    strcpy(mode_buf,rig_mode);
    }
  else if (strcmp("PKT",rig_mode) == 0) {
    strcpy(mode_buf,rig_mode);
    }
  else if (strcmp("CWN",rig_mode) == 0) {
    strcpy(mode_buf,"CW");
    filter_width = 1;
    }
  else if (strcmp("CWRN",rig_mode) == 0) {
    strcpy(mode_buf,"CWR");
    filter_width = 1;
    }
  else if (strcmp("DIGN",rig_mode) == 0) {
    strcpy(mode_buf,"DIG");
    filter_width = 1;
    }
  else if (rig_mode[0] == 0) {
    strcpy(mode_buf,"WFM");
    }

  // 表示用バッファに周波数とモードをセットしLCDの2行目に表示
  
  sprintf(disp_buf,"%03d.%03d.%03d  %3s", mhz, khz, hz, mode_buf);
  lcd.setCursor(0,0);
  lcd.print(disp_buf);

  // フィルタバンド幅をLCDの2行目に表示
  lcd.setCursor(13,1);
  if (filter_width == 1) {
    lcd.print("NAR");
    }
   else {
    lcd.print("   ");
    }
  
}

